package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.dto.UserArchivesAllocationDto;
import com.qmth.sop.business.bean.result.ProjectMonitorResult;
import com.qmth.sop.business.bean.result.ProjectMonitorUserResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBSopInfo;
import com.qmth.sop.business.mapper.TBSopInfoMapper;
import com.qmth.sop.business.service.ProjectMonitorService;
import com.qmth.sop.business.service.SysUserService;
import com.qmth.sop.business.service.TBUserArchivesAllocationService;
import com.qmth.sop.common.enums.CrmProcessEnum;
import com.qmth.sop.common.enums.SopRoleTypeEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ProjectMonitorServiceImpl extends ServiceImpl<TBSopInfoMapper, TBSopInfo> implements ProjectMonitorService {

    @Resource
    private SysUserService sysUserService;

    @Resource
    private TBUserArchivesAllocationService tbUserArchivesAllocationService;

    @Override
    public IPage<ProjectMonitorResult> pageProjectMonitor(Page<ProjectMonitorResult> page, Long serviceUnitId, Long customerId, CrmProcessEnum process) {
        SysUser user = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(user.getId());
        List<String> processList = new ArrayList<>();
        if (process != null) {
            processList = getProcessList(process);
        }

        IPage<ProjectMonitorResult> pageProjectMonitor = baseMapper.pageProjectMonitor(new Page<>(page.getCurrent(), page.getSize()),
                serviceUnitId, customerId, process, processList, dpr);
        List<ProjectMonitorResult> monitorResultList = pageProjectMonitor.getRecords();
        for (ProjectMonitorResult monitorResult : monitorResultList) {
            monitorResult.setProcess(CrmProcessEnum.findCrmProcessByDesc(monitorResult.getTaskName()));
            List<UserArchivesAllocationDto> allocationDtoList = tbUserArchivesAllocationService.listUserArchivesAllocation(
                    monitorResult.getCrmDetailId());
            List<ProjectMonitorUserResult> coordinatorList = filterUserArchivesAllocation(allocationDtoList, SopRoleTypeEnum.REGION_COORDINATOR);
            monitorResult.setRegionCoordinator(!coordinatorList.isEmpty() ? coordinatorList.get(0) : null);
            List<ProjectMonitorUserResult> managerList = filterUserArchivesAllocation(allocationDtoList, SopRoleTypeEnum.PROJECT_MANAGER);
            monitorResult.setProjectManager(!managerList.isEmpty() ? managerList.get(0) : null);
            monitorResult.setEngineerList(filterUserArchivesAllocation(allocationDtoList, SopRoleTypeEnum.ENGINEER));
        }

        return pageProjectMonitor;
    }

    private List<ProjectMonitorUserResult> filterUserArchivesAllocation(List<UserArchivesAllocationDto> allocationDtoList, SopRoleTypeEnum roleType) {
        List<ProjectMonitorUserResult> monitorResultList = new ArrayList<>();
        List<UserArchivesAllocationDto> filterUserList = allocationDtoList.stream().filter(user -> user.getSopRoleType().equals(roleType)).collect(
                Collectors.toList());
        for (UserArchivesAllocationDto dto : filterUserList) {
            ProjectMonitorUserResult userResult = new ProjectMonitorUserResult();
            BeanUtils.copyProperties(dto, userResult);
            monitorResultList.add(userResult);
        }
        return monitorResultList;
    }

    private List<String> getProcessList(CrmProcessEnum process) {
        List<String> processList = new ArrayList<>();
        if (process == CrmProcessEnum.FINISH) {
            return processList;
        }
        String[] descArr = process.getDesc().split(",");
        return Arrays.asList(descArr);
    }

}
